/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.modulUhrzeit;

import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.testsys.lib.MyLogger;
import de.bsvrz.sys.testsys.testsys.IBeendePlugin;
import de.bsvrz.sys.testsys.testsys.IGibZugriff;
import de.bsvrz.sys.testsys.testsys.IMeldungFehler;

/**
 * berprfer Plugin Uhrzeit; schickt mintlich eine Meldung der Form <br>
 *  Es ist hh:mm Uhr <br>
 * an die BetriebsmeldungsVerwaltung
 * @author Chriesten
 *
 */
public class ModulUhrzeit extends Thread implements IGibZugriff, IBeendePlugin
{
  /**
   * Aktuelle Verbindung zum Datenverteiler.
   */
  private ClientDavInterface m_davConnection;
  
  /**
   * Rueckrufobjekt zum Schreiben der Betriebsmeldungen
   */
  IMeldungFehler m_MeldungFehler = null;
  
  /**
   * Flag; soll sich der Thread beenden?
   */
  private Boolean m_beenden = false;

  // ---------------------------------------------------
  // Konstruktor
  // ---------------------------------------------------
  public ModulUhrzeit()
  {
    MyLogger.logln( "VERLAUF", "Konstruktur ohne Parameter " + this.getClass().getName());
    this.start();
  }

  public ModulUhrzeit(String args[])
  {
    MyLogger.logln( "VERLAUF", "Konstruktur mit Parameterarray " + this.getClass().getName());
    for (String arg : args)
    {
      MyLogger.logln("TT", arg);
    }
    //this.start();
  }
  // ---------------------------------------------------
  
  /**
   * Warte, bis zur nchsten vollen Sekunde sec
   */
  private void warteStart(String sec)
  {
    SimpleDateFormat sdf = new SimpleDateFormat("ss");
    String ss = sdf.format(new Date());
    while ( !ss.equals( sec ) )
    {
      ss = sdf.format(new Date());
      try
      {
        Thread.sleep( 800 );
      }
      catch ( InterruptedException e )
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    return;
  }
  
  /**
   * liefert aktuelle Uhrzeit in der Form hh:mm
   * @return aktuelle Uhrzeit, siehe obige Beschreibung
   */
  private String gibUhrzeit()
  {
    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
    String uhrzeit = sdf.format(new Date());

    return(uhrzeit);
  }

  /**
   * Schicke Betriebsmeldung aktuelle Uhrzeit. Format: "Es ist hh:mm Uhr!".
   */
  private void meldeUhrzeit()
  {
    String meldung = "Es ist "+gibUhrzeit()+" Uhr";
    if ( m_MeldungFehler != null )
    {
      m_MeldungFehler.fehlerGefunden( meldung, new Integer(1), this.getClass().getName());
    }
  }
  
  /** 
   * Wrapper Funktion fuer wait()
   * 
   * @param auszeit Wartezeit in ms
   */
  private void mywait(long auszeit) 
  { 
    try
    {
      wait(auszeit);
    }
    catch ( InterruptedException e )
    {
      System.err.println("NTPinfoPoller.mywait: InterruptedException Caught");
      //m_interrupted = true;
    }
    catch ( Exception e )
    {
      System.err.println("NTPinfoPoller.mywait: Exception gefangen");
      System.err.println(e.getMessage());
      e.printStackTrace();
    }
    return;
  } 
  
  /**
   * Meldet mintlich die Uhrzeit an die BetriebsMeldungsVerwaltung
   */
  @Override 
  public synchronized void run() 
  { 
    MyLogger.logln("VERLAUF", "Der Thread de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit wurde gestartet!" );

    meldeUhrzeit();
    warteStart("59");
    while(!m_beenden)
    {
      meldeUhrzeit();
      mywait(60000);
    }

    MyLogger.logln("VERLAUF", "Der Thread de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit wurde beendet!" );
  } 

  // -------------------------------------------------------------------
  // Methoden der Schnittstelle GibZugriff
  // -------------------------------------------------------------------
  /**
   * siehe {@link IGibZugriff}{@link #steuereUeberwachung(IMeldungFehler, ClientDavInterface)}
   */
  public void steuereUeberwachung( IMeldungFehler pluginVerwalter, ClientDavInterface connection )
  {
    m_MeldungFehler = pluginVerwalter;
    m_davConnection = connection;
  }
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  // Methode der Schnittstelle IBeendePlugin
  // -------------------------------------------------------------------
  /**
   * Stoppt den vom Plugin gestarteten Thread
   * siehe {@link IBeendePlugin}{@link #beenden()}.
   */
  public void beenden()
  {
    m_beenden = true;
    MyLogger.logln( "TTT_BEENDEN", "Plugin" + this.getClass().getName() + " soll sich beenden." );
  }
  // -------------------------------------------------------------------

}
