/**
 * 
 */
package de.bsvrz.sys.testsys.testsys;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.testsys.testsys.UeberpruefungSystemApp;
import de.bsvrz.sys.testsys.testsys.libBmLesen.EmpfaengerBm;

/**
 * JUNIT Tests fr den blackBox Test Ueberpreufung System.
 * (== white Box Test fr die main Routine). <br>
 * Arbeitsweise des Tests: <br> 
 * 1. Ueberpruefung System wird mit einem test Plugin gestartet. Das test Plugin erzeugt defnierte berprfer Meldungen. <br>
 * 2. DaV Empfngerprogramm wird gestartet. Es liest DaV Betriebsmeldungen. <br>
 * 3. Wurden innerhalb einer vorgegebenen Zeit (60sec) alle erwarteten Meldungen gelesen, war der Test erfolgreich, ansonsten fehlerhaft. <br>
 * <br>
 * Der Test prft die Erzeugung eines Plugins. <br> 
 * Der Test berprft die Schnittstellen IGibZugriff und IMeldungFehler, 
 * sowie die gesamte Verarbeitung der ber die Schnittstelle empfangenen Daten bis zur
 * Speicherung im DaV. <br>
 * Durch diesen Test wird die berprfung der Teilmoduln Verwaltung und berprfer abgedeckt.
 *
 */
public class UeberpruefungSystemAppTest
{
  /**
   * Ausgabe Logger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Pid, zum lesen der Betriebsmeldungen (typ.informationsKanal)
   */
  private String _objPidBm = null;

  /**
   * DaV Verbindung zum lesen der Betriebsmeldungen
   */
  public ClientDavInterface _connection = null;

  /**
   * JUNIT Test, oder mit java main gestartet?
   */
  private Boolean _isJUnitTest = true;

  /**
   * Zeitraum, bis alle erwarteten Betriebsmeldungen gelesen sein mssen
   */
  Long _warteLeseErgMs = null;


  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  
  /**
   * Initialisierungsroutine, liefert Programmargumente (Main-Args), mit denen  
   * das Programm ausgefhrt werden soll
   * @return "Main Argumente"
   */
  private String[] initArgs(Boolean parameterAusgabe)
  {
    _objPidBm = "betriebsMeldung.informationsKanal";
    _warteLeseErgMs = 60000L;
    
    String[] myargs = new String[10];
    int argno = 0;

    // globaler DAV
    String myDavServer = System.getenv( "DATENVERTEILER" ); // rechner:port
    if ( myDavServer == null )
    {
      // myDavServer = "192.0.1.128:8085";
      myDavServer = "localhost:8083";
      String myMess = "$DATENVERTEILER nicht gesetzt. Verwende "+myDavServer;
      System.out.println( myMess );
    }
    myargs[argno++]="-datenverteiler="+myDavServer;

    myargs[argno++]="-debugLevelStdErrText=FINE";
    myargs[argno++]="-benutzer=Tester";
    // myargs[argno++]="-authentifizierung=C:/Datenverteiler/skripte-dosshell/passwd";
    myargs[argno++]="-authentifizierung=./passwd";
    // myargs[argno++] = "-modLog=C:/temp/TestTestSys/log<UHRZEIT>.log::6::BM|ERROR";
    myargs[argno++] = "./log<UHRZEIT>.log::6::BM|ERROR";

    // plugins
    String pluginFuerTest = "de.bsvrz.sys.testsys.testsys.testPlugin.PluginFuerTest";
    myargs[argno++] = "-ueberpruefer=" + pluginFuerTest;
    
    // Groesse aufs notwendige vermindern
    String[] myargs2 = new String[argno];
    for (int i = 0; i < argno; i++)
    {
      myargs2[i] = myargs[i];
    }
    myargs = null;
    myargs = myargs2;

    if ( parameterAusgabe )
    {
      // Parameter ausgeben
      System.out.println("---------------------------------------");
      for (String arg : myargs)
      {
        System.out.println(arg);
      }
      System.out.println("---------------------------------------");
    }
    
    return myargs2;
  }

  /**
   * Test method for {@link UeberpruefungSystemApp#main(String[])}. <br>
   * Einziger Test in dieser Klasse. Spezifikation entspricht der Klassenbeschreibung.
   */
  @Test
  public void testMain()
  {
    // -------------------------------------------------
    // Ueberpruefung Systen (mit Testplugin) starten
    // und Initialisierung abwarten
    // -------------------------------------------------
    String[] myargs = initArgs(true);
    UeberpruefungSystemApp.main( myargs );
    try
    {
      Thread.sleep( 10000 );
    }
    catch ( InterruptedException e )
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    _connection = UeberpruefungSystemApp._statConnection;
    // -------------------------------------------------

    // -------------------------------------------
    // erwartete Betriebsmeldungen lesen
    // -------------------------------------------

    EmpfaengerBm empf = new EmpfaengerBm(_connection);
    Boolean bret = empf.testPruefeBm(_objPidBm, _warteLeseErgMs);
    
    String mess = "Gesamttest Ueberpruefung System <<nicht>> erfolgreich.";
    mess = mess.replaceFirst( "<<nicht>>", bret?"":"nicht");
    // -------------------------------------------

    
    // -------------------------------------------
    // Ergebnis auswerten
    // -------------------------------------------
    if ( _isJUnitTest )
      assertTrue(mess, bret == true);
    
    if ( bret )
    {
      _debug.finest( mess );
      System.out.println(mess);
    }
    else
    {
      _debug.error( mess );
    }
    
    if ( !_isJUnitTest )
    {
      if ( bret ) 
        System.exit( 0 );
      else
        System.exit( 1 );
    }
  }

  /**
   * Test Methode. Kann anstelle JUNIT Test in Scripts eingebaut werden.
   * Liefert Exitwert 0 bei erfolgreichem Test.
   * 
   * @param args ignoriert. Die Argumente fr den Test liefert die Methode initArgs.
   */
  public static void main(String args[])
  {
    // Ueberpruefung System initialisieren
    UeberpruefungSystemAppTest te = new UeberpruefungSystemAppTest();
    te._isJUnitTest = false;
    te.testMain();
  }
}
