/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.testsys.libBmLesen;


import java.net.InetAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.testsys.lib.MyLogger;

/**
 * Funktionen zum Empfang nicht konfigurierender DaV Daten.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/02/09 14:55:51 $ / ($Author: chriesten $)
 *
 * @author Dambach-Werke GmbH
 * @author Chriesten
 *
 */
public class EmpfaengerBm implements ClientReceiverInterface
{
  /**
   * Ausgabe Logger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Objekt zur Synchrnisierung 
   * Warten auf Ergebnis Betriebsmeldungsverwaltung und
   * Lieferung des Ergebnisses
   */
  private Object _pruefeBmSync = new Object();
  
  /**
   * maximale Wartezeit bei synchronen Anfragen
   */
  final long EMPFANGS_TIMEOUT_SYNC = 20000; 

  /**
   * Flag zur Beendigung der asynchronen Endlosschleife
   */
  private Boolean m_beenden = false;
  
  /**
   * Aktuelle Verbindung zum Datenverteiler.
   */
  private ClientDavInterface m_connection = null;

  /**
   * bergebenes Datenmodell
   */
  private DataModel m_dataModel = null;
  
  /**
   * Konstruktor
   * @param connection aktuelle DaV Verbindung
   */
  public EmpfaengerBm( ClientDavInterface connection )
  {
    m_connection = connection;
    m_dataModel = m_connection.getDataModel();
  }

  /**
   * Empfangsanmeldung am DaV. Hllfunktion zu {@link #anmeldeLeseDav(String, String, String, Short, ReceiverRole)}. 
   * 
   *  @param objPid Objekt 
   *  @param atgPid Attributgruppe
   *  @param aspPid Aspekt
   *  @param sv Simulationsvariante (null: keine Simulation)
   *  
   *  @return true: Anmeldung ok
   */
  public boolean anmeldeObjekt( String objPid, String atgPid, String aspPid, Short sv )
  {
    boolean bret = false;
    
    if ( sv == null )
      sv = DataDescription.NO_SIMULATION_VARIANT_SET;

    ReceiverRole role = ReceiverRole.receiver();

    bret = anmeldeLeseDav( objPid, atgPid, aspPid, sv, role );
    return (bret);
  }

  /**
   * Abmeldung DaV Empfang. Hllfunktion.
   * 
   *  @param objPid Objekt 
   *  @param atgPid Attributgruppe
   *  @param aspPid Aspekt
   *  @param sv Simulationsvariante (null: keine Simulation)
   */
  public void abmeldeObjekt( String objPid, String atgPid, String aspPid, Short sv )
  {
    if ( sv == null )
      sv = DataDescription.NO_SIMULATION_VARIANT_SET;

    abmeldeLeseDav( objPid, atgPid, aspPid, sv );
    
    return;
  }

  
  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler zum Lesen anmeldet, 
   * Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objPid Pid des Objekts
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   * @param role Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   * @param sv SimulationsVariante
   * 
   * @return true: ok, false: nicht ok
   */
  public boolean anmeldeLeseDav( String objPid, String atgPid, String aspPid, Short sv, 
                                  ReceiverRole role )
  {
    sv = ((sv!=null) ? sv : DataDescription.NO_SIMULATION_VARIANT_SET);

    String buffer = "Anmelden am DaV (Lesen): " + objPid + " " + atgPid + "  " + aspPid + "  " + sv + " role=" + role.toString();
    MyLogger.logln( "VERLAUF", buffer );

    ClientDavInterface verb = m_connection;

    try
    {
      AttributeGroup atg = verb.getDataModel().getAttributeGroup( atgPid );
      Aspect asp = verb.getDataModel().getAspect( aspPid );

      DataDescription dd = new DataDescription( atg, asp, sv );

      SystemObject obj;
      if ( objPid.matches( "[0-9]*" ) )
      { // nur Zahlen == id
          Long objId = new Long(objPid );
          obj = verb.getDataModel().getObject( objId );
      }
      else if ( objPid.equalsIgnoreCase( "LocalDav") )
      {
        obj = m_connection.getLocalDav();
      }
      else // pid
      {
        obj = verb.getDataModel().getObject( objPid );
      }

      verb.subscribeReceiver( this, obj, dd, ReceiveOptions.normal(), role );
    }
    catch ( Exception e1 )
    {
      MyLogger.logln( "ERROR", "subscribeReceiver:" + buffer );
      MyLogger.logln( "ERROR", e1.getMessage() );
      MyLogger.log( "ERROR", "" );
      e1.printStackTrace( MyLogger.get_out() );
      return (false);
    }
    return (true);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler vom Lesen abmeldet, 
   * Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objPid Pid oder Id des Objekts
   * @param atgPid Attributgruppe die abgemeldet werden soll
   * @param aspPid Apekt der abgemeldet werden soll
   * @param sv Simualtionsvariante
   * @return true:ok         
   */
  public Boolean abmeldeLeseDav( String objPid, String atgPid, String aspPid, Short sv )
  {
    sv = ((sv!=null) ? sv : DataDescription.NO_SIMULATION_VARIANT_SET);

    String buffer = "Abmelden vom DaV (Lesen): " + objPid + " " + atgPid + "  " + aspPid;
    MyLogger.logln( "VERLAUF", buffer );

    ClientDavInterface verb = m_connection;

    try
    {
      AttributeGroup atg = verb.getDataModel().getAttributeGroup( atgPid );
      Aspect asp = verb.getDataModel().getAspect( aspPid );

      DataDescription dd = new DataDescription( atg, asp, sv );

      SystemObject obj;
      if ( objPid.matches( "[0-9]*" ) )
      { // nur Zahlen == id
          Long objId = new Long(objPid );
          obj = verb.getDataModel().getObject( objId );
      }
      else // pid
      {
        obj = verb.getDataModel().getObject( objPid );
      }

      verb.unsubscribeReceiver( this, obj, dd );
    }
    catch ( Exception e1 )
    {
      MyLogger.logln( "ERROR", "unsubscribeReceiver:" + buffer );
      MyLogger.logln( "ERROR", e1.getMessage() );
      e1.printStackTrace();
      return(false);
    }
    return(true);
  }

  
  // -------------------------------------------------
  // Schnittstelle ClientReceiverInterface
  // -------------------------------------------------
  /**
   * see {@link ClientReceiverInterface#update(ResultData[])}.
   */
  public void update( ResultData[] results )
  {
    for (int i = 0; i < results.length; i++)
    {
      ResultData rdat = results[i];
      
      // Test Ausgeben
      ausDavInfo( rdat );
      
      // verteilen
      // m_davInfoKonsumenten.verteileResultData( rdat );
      neueBetriebsInforamtion( rdat );
    }

    synchronized(this)
    {
      notify();
    }
  }
  // -------------------------------------------------

  
  /**
   * Gibt rdat Informationen aus ; Debug Routine.
   * <p>
   * @param rdat DaV Ergebnis Daten Satz  
   */
  public void ausDavInfo( ResultData rdat )
  {
    String objPid = rdat.getObject().getPid();
    String atgPid = rdat.getDataDescription().getAttributeGroup().getPid();
    String aspPid = rdat.getDataDescription().getAspect().getPid();
    short sv = rdat.getDataDescription().getSimulationVariant();

    Boolean dataAvailable = false;
    Boolean sourceAvailable = rdat.isSourceAvailable();
    if ( sourceAvailable )
      dataAvailable = !(rdat.isNoDataAvailable());
    
    String mess = "ResultData: ";
    mess += " ,objpid="+objPid;
    mess += " ,atg="+atgPid;
    mess += " ,asp="+aspPid;
    mess += " ,dat="+dataAvailable;
    mess += " ,src="+sourceAvailable;
    mess += " ,sv="+sv;
    
    String meldungsTextS = null;
    // notwendig: beim ersten Aufruf direkt nach Neustart des Dav
    // geht folgendes schief:
    // meldungsTextS = information.getItem( "MeldungsText" ).asTextValue().getValueText(); 

    try
    {
      Data information = rdat.getData();
      meldungsTextS = information.getItem( "MeldungsText" ).asTextValue().getValueText(); 
    }
    catch ( NullPointerException e )
    {
      meldungsTextS = "Liefert keinen Wert: information.getItem( \"MeldungsText\" ).asTextValue().getValueText()";
      _debug.warning( meldungsTextS );
    }

    _debug.fine(mess);
    _debug.fine(meldungsTextS);
  }
  
  /**
   * Aufbau zu testender Betriebsmeldungen
   */
  private String _bmTestMeldung = "Test Plugin [1-3]";

  /**
   * wird auf true gesetzt, soblad die 3 erwarteten Betriebsmeldungen
   *  Test Plugin 1
   *  Test Plugin 2
   *  Test Plugin 3
   * empfangen wurden
   */
  public Boolean _alleTestBetrMeldungenEmpfangen = false;

  /**
   * Marker. Enthlt Elemente, die als Betriebsmeldung empfangen wurden
   */
  private HashMap<String, String>gefunden = new HashMap<String, String>(); 

  
  /**
   * Liest Betriebsmeldungsstrings. Sobald alle 3 Betriebsmeldungen
   * "Test Plugin 1" ... "Test Plugin 3" empfangen wurden,
   * wird _alleTestBetrMeldungenEmpfangen auf true gesetzt 
   *    
   * @param rdat DaV Ergebnis Daten Satz  
   */
  public void neueBetriebsInforamtion( ResultData rdat )
  {
    // System.out.println("Test(1)DavInfoKonsumenten: neu DaV Info erhalten...");
    String objPid = rdat.getObject().getPid();
    String atgPid = rdat.getDataDescription().getAttributeGroup().getPid();
    String aspPid = rdat.getDataDescription().getAspect().getPid();
    short sv = rdat.getDataDescription().getSimulationVariant();

    if (  !(atgPid.equals("atg.betriebsMeldung")) || 
          ! (aspPid.equals("asp.information")) )
    {
      // not for us
      return;
    }
    
    Boolean dataAvailable = !(rdat.isNoDataAvailable());
    if ( !(rdat.isSourceAvailable()) )
    {
      String mess = "Keine Quelle fr "+ objPid;
      _debug.error( mess );
      return;
    }
    else if ( rdat.isNoDataAvailable() )
    {
      String mess = "Keine Daten fr "+ objPid;
      _debug.error( mess );
      return;
    }

    String bmPid = rdat.getObject().getPid();
    Data information = rdat.getData();

    long applikationsID = information.getItem( "ApplikationsID" ).asUnscaledValue().longValue(); 
    String applikationsKennung = information.getItem( "ApplikationsKennung" ).asTextValue().getValueText(); 
    String meldungsText = information.getItem( "MeldungsText" ).asTextValue().getValueText(); 

    Data urlasser = information.getItem( "Urlasser" );
    String veranlasser = urlasser.getItem( "Veranlasser" ).asTextValue().getValueText();

    System.out.println( "==========meldText= "+meldungsText);

    // 
    if ( meldungsText.matches( _bmTestMeldung ) )
    {
      _debug.finest( "==========GEFUNDEN==XXX== "+meldungsText);
      String indS = meldungsText.replaceFirst( "([^1-3]*)([1-3])", "$2" );
      if ( !(gefunden.containsKey( indS )) )
      {
        gefunden.put( indS, indS );
        _debug.finest( "==========GEFUNDEN"+indS+" "+gefunden.size());
      }
      if ( gefunden.size() == 3)
      {
        _alleTestBetrMeldungenEmpfangen = true;
        // wecke wait in testPruefeBm
        synchronized ( _pruefeBmSync )
        {
          _pruefeBmSync.notify();
        }
      }
    }
    return;
  }

  
  /**
   * Testroutine fr leseBmAsync()
   * 
   * @param objPid Pid des Informationskanals fuer Betriebsmeldungen
   * @param warteMs spaetestens nach dieser Zeit muessen die erwarteten Betriebsmeldungen
   *    angekommen sein
   * @return true ok:erwartete BMs empfangen, false: BMs nicht empfangen, Fehler
   */
  public Boolean testPruefeBm(String objPid, long warteMs)
  {
    // anmelden
    if ( objPid==null )
    {
      // objPid = connection.getDataModel().getConfigurationAuthorityPid();
      objPid = "betriebsMeldung.informationsKanal";
    }
    String atgPid = "atg.betriebsMeldung"; 
    String aspPid = "asp.information";
    
    anmeldeObjekt(objPid, atgPid, aspPid, null);

    // warten
    synchronized(_pruefeBmSync)
    {
      try
      {
        _pruefeBmSync.wait(warteMs);  
      }
      catch(InterruptedException e)
      {
        _debug.finest( "pruefeBm, interrupted Excption");
      }
    }
    abmeldeObjekt(objPid, atgPid, aspPid, null);
    _debug.config( "pruefeBm fertig: "+_alleTestBetrMeldungenEmpfangen);
    return(_alleTestBetrMeldungenEmpfangen);
  }


  /**
   * Test und Beispielroutine zum lesen und testen der Betriebsmeldungen
   */
  public static void testLeseBm(String objPid, ClientDavInterface connection)
  {
    long warteMs = 20000;

    EmpfaengerBm empf = new EmpfaengerBm(connection);
    empf.testPruefeBm( objPid, warteMs);
  }
}
