/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.testsys.testsys.testPlugin;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.testsys.lib.MyLogger;
import de.bsvrz.sys.testsys.testsys.IBeendePlugin;
import de.bsvrz.sys.testsys.testsys.IGibZugriff;
import de.bsvrz.sys.testsys.testsys.IMeldungFehler;

/**
 * Test Plugin fr JUnit Test: 
 * schickt zyklisch die Meldungen 
 *  Test Plugin 1
 *  Test Plugin 2
 *  Test Plugin 3
 * an die BetriebsmeldungsVerwaltung
 * und beendet sich dann
 * @author Chriesten
 *
 */
public class PluginFuerTest extends Thread implements IGibZugriff, IBeendePlugin
{
  /**
   * Ausgabe Logger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Aktuelle Verbindung zum Datenverteiler.
   */
  private ClientDavInterface m_davConnection;
  
  /**
   * Rueckrufobjekt zum Schreiben der Betriebsmeldungen
   */
  IMeldungFehler m_MeldungFehler = null;
  
  /**
   * Flag; soll sich der Thread beenden?
   */
  private Boolean m_beenden = false;

  // ---------------------------------------------------
  // Konstruktor
  // ---------------------------------------------------
  public PluginFuerTest()
  {
    MyLogger.logln( "VERLAUF", "Konstruktur ohne Parameter " + this.getClass().getName());
    this.start();
  }

  public PluginFuerTest(String args[])
  {
    MyLogger.logln( "VERLAUF", "Konstruktur mit Parameterarray " + this.getClass().getName());
    for (String arg : args)
    {
      MyLogger.logln("TT", arg);
    }
    //this.start();
  }
  // ---------------------------------------------------
  
  /** 
   * Wrapper Funktion fuer wait()
   * 
   * @param auszeit Wartezeit in ms
   */
  private void mywait(long auszeit) 
  { 
    try
    {
      wait(auszeit);
    }
    catch ( InterruptedException e )
    {
      _debug.warning( "NTPinfoPoller.mywait: InterruptedException Caught");
      //m_interrupted = true;
    }
    catch ( Exception e )
    {
      String errMess = "NTPinfoPoller.mywait: Exception gefangen";
      errMess += "\n" + e.getMessage();
      _debug.warning( errMess );
      System.err.println(errMess);
    }
    return;
  } 
  
  /**
   * schicke 10 sekndlich die Meldungen
   * Test Plugin 1
   * Test Plugin 2
   * Test Plugin 3
   * 
   */
  @Override 
  public synchronized void run() 
  { 
    int runs = 0;
    
    mywait( 10000 );
    while(!m_beenden && runs<60)
    {
      meldeTest();
      mywait(5000);
    }

    MyLogger.logln("VERLAUF", "Der Thread de.bsvrz.sys.testsys.modulUhrzeit.ModulUhrzeit wurde beendet!" );
  } 

  /**
   * Schicke Test Betriebsmeldung. Format: "Test Plugin 1"..."Test Plugin 3"
   */
  private void meldeTest()
  {
    if ( m_MeldungFehler != null )
    {
      for (int i = 1; i <= 3; i++)
      {
        String meldung = "Test Plugin "+ i;
        m_MeldungFehler.fehlerGefunden( meldung, new Integer(1), this.getClass().getName());
      }
    }
  }

  
  
  // -------------------------------------------------------------------
  // Methoden der Schnittstelle GibZugriff
  // -------------------------------------------------------------------
  /**
   * siehe {@link IGibZugriff}{@link #steuereUeberwachung(IMeldungFehler, ClientDavInterface)}
   */
  public void steuereUeberwachung( IMeldungFehler pluginVerwalter, ClientDavInterface connection )
  {
    m_MeldungFehler = pluginVerwalter;
    m_davConnection = connection;
  }
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  // Methode der Schnittstelle IBeendePlugin
  // -------------------------------------------------------------------
  /**
   * Stoppt den vom Plugin gestarteten Thread
   * siehe {@link IBeendePlugin}{@link #beenden()}.
   */
  public void beenden()
  {
    m_beenden = true;
    MyLogger.logln( "TTT_BEENDEN", "Plugin" + this.getClass().getName() + " soll sich beenden." );
  }
  // -------------------------------------------------------------------

}
